//
// AMIS30624 Micro-Stepper (actuator) 
// Arduino Library
//
//
// May 15, 2011 - initial release
// May 19, 2012 - revised types
//
// CC = BY NC SA
// http://embeddedcomputing.weebly.com
//

// Core library
#if defined(__AVR_ATmega328P__) || defined(__AVR_ATmega2560__) // Arduino specific
#include "WProgram.h"	
#elif defined(__32MX320F128H__) || defined(__32MX795F512L__) // chipKIT specific 
#include "WProgram.h"
#elif defined(__AVR_ATmega644P__) // Wiring specific
#include "Wiring.h"
#elif defined(__MSP430G2452__) || defined(__MSP430G2553__) || defined(__MSP430G2231__) // LaunchPad specific
#include "Energia.h"
#elif defined(MCU_STM32F103RB) || defined(MCU_STM32F103ZE) || defined(MCU_STM32F103CB) || defined(MCU_STM32F103RE) // Maple specific
#include "WProgram.h"	
#endif

// Other libraries
#include "Wire.h"
#include "I2C_Stepper.h"


// ---------------- Functions
// Writes value to address registry on device
static void _writeByteTo(int8_t device, uint8_t registry, uint8_t value) {
    Wire.beginTransmission(device); //start transmission to ACC 
    Wire.send(registry);             // send registry address
    Wire.send(value);               // send value to write
    Wire.endTransmission();         //end transmission
}

// Writes value to address registry on device
static void _writeTo(int8_t device, uint8_t registry, uint8_t number, uint8_t buffer[]) {
    Wire.beginTransmission(device); //start transmission to ACC 
    Wire.send(registry);        //sends address to write to
    
    for (uint8_t i=0; i<number; i++) {
        Wire.send(buffer[i]);                // send value to write
    }
    Wire.endTransmission();         //end transmission
}


// Reads number bytes starting from address registry on device into buffer array
static void _readFrom(int8_t device, uint8_t registry, uint8_t number, uint8_t buffer[]) {
    Wire.beginTransmission(device); //start transmission to ACC 
    Wire.send(registry);        // sends address to read from
    Wire.endTransmission();     // end transmission 
    
    Wire.beginTransmission(device); //start transmission to ACC
    Wire.requestFrom(device, number);    // request 6 bytes from ACC
    
    uint8_t i = 0;
    while(Wire.available())    //ACC may send less than requested (abnormal)
    { 
        buffer[i] = Wire.receive(); // receive a byte
        i++;
    }
    Wire.endTransmission(); //end transmission
}

static uint8_t _readByteFrom(int8_t device, uint8_t registry0) {
    Wire.beginTransmission(device);
    Wire.send(registry0);
    Wire.endTransmission();
    
    Wire.requestFrom(device, 2);
    while (Wire.available()<1) { 
        ; 
    }
    uint8_t result = Wire.receive();
    return result;
}


static int _readIntFrom(int8_t device, uint8_t registry0) {
    return _readByteFrom(device, registry0) + (_readByteFrom(device, registry0 + 1) << 8);
}




I2C_Stepper::I2C_Stepper(uint8_t n)
{
    _address=0x60;     // 7 bit address
    if (n==1) _address=0x61;
}


String I2C_Stepper::WhoAmI() {
    String s="";
    if (_address < 0x10) s="0";
    s = s + String(_address, 0x10) +"h ";
    s = s + "AMIS30624 Micro-Stepper (actuator)";
    return s ;
}



void I2C_Stepper::begin() {
    // Wake-up
    Wire.beginTransmission(_address); 
    Wire.send(0x81);             
    Wire.endTransmission();      
    resetDefault();
}

void I2C_Stepper::resetDefault() {
    Wire.beginTransmission(_address); 
    Wire.send(0x87);             
    Wire.endTransmission();      
}


void I2C_Stepper::setParameters()
{
    uint8_t buffer[7];
    
    buffer[0]=0xff; // 0xff
    buffer[1]=0xff; // 0xff
    buffer[2]=0x88; // iRun 3-0, iHold 3-0
    buffer[3]=0x81; // vMax 3-0, vMin 3-0
    buffer[4]=0x00; // secPos 10-8, shaft, acc 3-0
    buffer[5]=0x00; // secPos 7-0
    buffer[6]=0x0c; // 1, PWMfreq, 1, accShape, stepMode 1-0, 1, PWMJEn
    
    _writeTo(_address, 0x89, 7, buffer);
    
}


void I2C_Stepper::getStatus()
{
    uint8_t buffer[8];
    uint8_t i;
    
    _readFrom(_address, 0x81, 8, buffer); 
    // 0: address
    
    // data 1: i
    _iRun  = buffer[1]>>4;
    _iHold = buffer[1] & 0x0f;
    
    // data 2: v  
    _vMax  = buffer[2]>>4;
    _vMin  = buffer[2] & 0x0f;
    
    // data 3: definitions
    // acceleration, step mode, shaft, ...
    
    // data 4: error flags  
	switch (bitRead(buffer[4], 1) <<1 + bitRead(buffer[4], 0)) {
		case 0b00:		
			_testTemperatureInfo="normal";
			break;
		case 0b01:		
			_testTemperatureInfo="low";
			break;
		case 0b10:		
			_testTemperatureInfo="high";
			break;
		case 0b11:		
			_testTemperatureInfo="motor stopped";
			break;
			
	}
    _testThermalWarning = bitRead(buffer[4], 2); 
    _testThermalShutDown = bitRead(buffer[4], 3); 
    _testBatteryUnderVoltage = bitRead(buffer[4], 4); 
    _testElectricalDefect = bitRead(buffer[4], 5); 
    _testStepLoss = bitRead(buffer[4], 6); 
    _testDigitalVoltageReset = bitRead(buffer[4], 7); 
    
    // data 5: motion
    _testPumpFailure = bitRead(buffer[5], 0); 
    _testStall = bitRead(buffer[5], 1); 
    _testOverCurrent = bitRead(buffer[5], 3) | bitRead(buffer[5], 2); 
    _testOverCurrentX = bitRead(buffer[5], 3); 
    _testOverCurrentY = bitRead(buffer[5], 2); 
    _testReferenceReached = bitRead(buffer[5], 4); 
    _testMotion = buffer[5] >>5; 
    _testStop = ( (bitRead(buffer[5], 5)==0) && (bitRead(buffer[5], 6)==0) );
	
    _testError = ( (buffer[4] >0) || ((buffer[5] & 0b00011111) >0) ); 
    
    // data 6: 0xff
    // data 7: threeshold definitions
    
    //  Serial.print("\nGetFullStatus1\n");
    //  for (i=0; i<8; i++) {
    //    Serial.println(buffer[i], BIN);
    //  }
    
    for (i=0; i<8; i++) buffer[i]=0;
    //  Serial.print("\nGetFullStatus2\n");
    _readFrom(_address, 0xfc, 8, buffer); 
    
    //  for (i=0; i<8; i++) {
    //    Serial.println(buffer[i], BIN);
    //  }
    
    _actualPosition = (int16_t)(buffer[1] <<8) + buffer[2];
    _targetPosition = (int16_t)(buffer[3] <<8) + buffer[4];
    _securePosition = (buffer[6] & 0b111) + buffer[5];
    
    
}

void I2C_Stepper::goPosition(int16_t position)
{
    uint8_t buffer[4];
    
    buffer[0]=0xff; // 0xff
    buffer[1]=0xff; // 0xff
    buffer[2]= position >>8; // pos 15-8
    buffer[3]= position & 0x00ff; // pos 7-0
    
    _writeTo(_address, 0x8b, 4, buffer);
    delay(10);
}


void I2C_Stepper::resetPosition() {
    Wire.beginTransmission(_address); 
    Wire.send(0x86);             
    Wire.endTransmission();      
    
}


void I2C_Stepper::goSecurePosition() {
    Wire.beginTransmission(_address); 
    Wire.send(0x84);             
    Wire.endTransmission();      
}


void I2C_Stepper::softStop() {
    Wire.beginTransmission(_address); 
    Wire.send(0x8f);             
    Wire.endTransmission();      
}


void I2C_Stepper::hardStop() {
    Wire.beginTransmission(_address); 
    Wire.send(0x85);             
    Wire.endTransmission();      
}


void I2C_Stepper::runVelocity() {
    Wire.beginTransmission(_address); 
    Wire.send(0x97);             
    Wire.endTransmission();        
}

void I2C_Stepper::goReference() {
    uint8_t buffer[8];
    
    //
	resetPosition();	
	
    //Serial.print("Slow speed\n");
    // Slow speed
    buffer[0]=0xff; // 0xff
    buffer[1]=0xff; // 0xff
    buffer[2]=0x84; // iRun 3-0, iHold 3-0
    buffer[3]=0x41; // vMax 3-0, vMin 3-0
    buffer[4]=0x10; // secPos 10-8, shaft, acc 3-0
    buffer[5]=0x00; // secPos 7-0
    buffer[6]=0x0c; // 1, PWMfreq, 1, accShape, stepMode 1-0, 1, PWMJEn
    
    _writeTo(_address, 0x89, 7, buffer);
    
    //Serial.print("Go far\n");
    // Go far  
    do {
        //        buffer[5]=0xff;
        goPosition(10000);
        //runVelocity(); deprecated
        // Check ESW  
        _readFrom(_address, 0x81, 8, buffer); 
        // Serial.println(buffer[5], BIN);
        
        // test motion
        if ( buffer[5] >> 5==0 ) {
            resetPosition();
            goPosition(10000);            
        }        
    } 
    // test external switch
    while  (!bitRead(buffer[5], 4));
    
    
    // Hard stop
    hardStop();
    //Serial.print("Hard stop\n");
    
    resetPosition();
}










